from LigParGen.BOSSReader import BOSSReader, CheckForHs
from LigParGen.BOSS2OPENMM import mainBOSS2OPM
from LigParGen.BOSS2CHARMM import mainBOSS2CHARMM
from LigParGen.BOSS2GMX import mainBOSS2GMX
from LigParGen.BOSS2XPLOR import mainBOSS2XPLOR
from LigParGen.BOSS2Q import mainBOSS2Q
from LigParGen.CreatZmat import GenMolRep
import argparse
import pickle
import os
# from shutil import which

def main():

    parser = argparse.ArgumentParser(
        prog='Converter.py',
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description="""
    SCRIPT TO CONVERT WRITE OPENMM PDB AND XML FILES 
    FROM BOSS ZMATRIX
    Created on Mon Feb 15 15:40:05 2016
    @author: Leela S. Dodda leela.dodda@yale.edu
    @author: William L. Jorgensen Lab 
    
    if using BOSS Zmat
    Usage: python Converter.py -z phenol.z -r PHN -c 0 

    if using MOL file 
    Usage: python Converter.py -m phenol.mol -r PHN -c 0

    if using PDB file 
    Usage: python Converter.py -p phenol.pdb -r PHN -c 0
    
    if using BOSS SMILES CODE 
    Usage: python Converter.py -s 'c1ccc(cc1)O' -r PHN -c 0 
    
    REQUIREMENTS:
    BOSS (need to set BOSSdir in bashrc and cshrc)
    Preferably Anaconda python with following modules
    pandas 
    argparse
    numpy
    RDKit for using with SMILES code
    """
    )
    parser.add_argument(
        "-r", "--resname", help="Residue name from PDB FILE", type=str)
    parser.add_argument(
        "-s", "--smiles", help="Paste SMILES code from CHEMSPIDER or PubChem", type=str)
    parser.add_argument(
        "-m", "--mol", help="Submit MOL file from CHEMSPIDER or PubChem", type=str)
    parser.add_argument(
        "-p", "--pdb", help="Submit PDB file from CHEMSPIDER or PubChem", type=str)
    parser.add_argument(
        "-o", "--opt", help="Optimization or Single Point Calculation", type=int, choices=[0, 1, 2, 3])
    parser.add_argument("-c", "--charge", type=int,
                        choices=[0, -1, 1, -2, 2], help="0: Neutral <0: Anion >0: Cation ")
    parser.add_argument(
        "-l", "--lbcc", help="Use 1.14*CM1A-LBCC charges instead of 1.14*CM1A", action="store_true")
    args = parser.parse_args()

    convert(**vars(args))

def convert(**kwargs):

    # set the default values
    options = {
            'opt' : 0,
            'smiles' : None,
            'zmat' : None, 
            'charge' : 0,
            'lbcc' : False,
            'mol' : None,
            'resname' : 'UNK',
            'pdb' : None }

    # update the default values based on the arguments
    options.update(kwargs)

    # set the arguments that you would used to get from argparse
    opt = options['opt']
    smiles = options['smiles']
    zmat = options['zmat']
    charge = options['charge']
    lbcc = options['lbcc']
    resname = options['resname']
    mol = options['mol']
    pdb = options['pdb']

    optim = opt
    clu = False
    #charge = charge
    lbcc = False

    # assert (which('obabel')
            # is not None), "OpenBabel is Not installed or \n the executable location is not accessable"
    if os.path.exists('/tmp/' + resname + '.xml'):
        os.system('/bin/rm /tmp/' + resname + '.*')
    if lbcc:
        if charge == 0:
            lbcc = True
        else:
            lbcc = False
            print(
                '1.14*CM1A-LBCC is only available for neutral molecules\n Assigning unscaled CM1A charges')

    if smiles != None:
        os.chdir('/tmp/')
        smifile = open('%s.smi' % resname, 'w+')
        smifile.write('%s' % smiles)
        smifile.close()
        GenMolRep('%s.smi' % resname, optim, resname, charge)
        mol = BOSSReader('%s.z' % resname, optim, charge, lbcc)
    elif mol != None:
        os.system('cp %s /tmp/' % mol)
        os.chdir('/tmp/')
        GenMolRep(mol, optim, resname, charge)
        mol = BOSSReader('%s.z' % resname, optim, charge, lbcc)
    elif pdb != None:
        os.system('cp %s /tmp/' % pdb)
        os.chdir('/tmp/')
        GenMolRep(pdb, optim, resname, charge)
        mol = BOSSReader('%s.z' % resname, optim, charge, lbcc)
        clu = True
    assert (mol.MolData['TotalQ']['Reference-Solute'] ==
            charge), "PROPOSED CHARGE IS NOT POSSIBLE: SOLUTE MAY BE AN OPEN SHELL"
    assert(CheckForHs(mol.MolData['ATOMS'])
           ), "Hydrogens are not added. Please add Hydrogens"

    pickle.dump(mol, open(resname + ".p", "wb"))
    mainBOSS2OPM(resname, clu)
    print('DONE WITH OPENMM')
    mainBOSS2Q(resname, clu)
    print('DONE WITH Q')
    mainBOSS2XPLOR(resname, clu)
    print('DONE WITH XPLOR')
    mainBOSS2CHARMM(resname, clu)
    print('DONE WITH CHARMM/NAMD')
    mainBOSS2GMX(resname, clu)
    print('DONE WITH GROMACS')
    os.remove(resname + ".p")
    mol.cleanup()

if __name__ == "__main__":
  
    main()



"""
SCRIPT TO CONVERT WRITE CHARMM RTF AND PRM FILES 
FROM BOSS ZMATRIX
Created on Mon Feb 15 15:40:05 2016
@author: Leela S. Dodda leela.dodda@yale.edu
@author: William L. Jorgensen Lab 

Usage: python OPM_Routines.py -z phenol.z -r PHN
REQUIREMENTS:
BOSS (need to set BOSSdir in bashrc and cshrc)
Preferably Anaconda python with following modules
pandas 
argparse
numpy
"""

from LigParGen.BOSSReader import bossPdbAtom2Element,bossElement2Mass,ucomb
import pickle
import pandas as pd
import numpy as np


def retDihedImp(df):
    odihed = []
    if np.sum([df['V' + str(pot)] for pot in range(1, 5)]) != 0.0:
        for pot in range(1, 5):
            if (df['V' + str(pot)] != 0.0):
                odihed.append('IMPRoper %s %4.5f %d %4.5f \n' % (df['NAME'].replace(
                    "-", "   "), df['V' + str(pot)], pot, 180.00 * abs(pot % 2 - 1)))
    else:
        pot = 2
        odihed.append('IMPRoper %s %4.5f %d %4.5f \n' % (df['NAME'].replace(
            "-", "   "), df['V' + str(pot)], pot, 180.00 * abs(pot % 2 - 1)))
    return (odihed)


def retDihed(df):
    odihed = []
    for pot in range(1, 5):
        odihed.append('DIHEdral %s           %4.5f %d %4.5f \n' % (df['NAME'].replace(
            "-", "   "), df['V' + str(pot)], pot, 180.00 * abs(pot % 2 - 1)))
    return (odihed)


def Boss2CharmmRTF(num2typ2symb, Qs, resid, bnd_df, angs,props,imps):
    charges = [float(Qs[i][1]) for i in range(len(Qs))]
    rtf = open(resid + '.top', 'w+')
    rtf.write('Remarks LigParGen generated XPLOR-TOP file for Bonvin group (by Leela Dodda)\n')
    rtf.write('\nset echo=false end\n')
    rtf.write('\nautogenerate angles=True dihedrals=True end\n')
    rtf.write('{ atomType  mass }\n')
    Mass = ['MASS %s %3.4f \n' % (num2typ2symb[i][2], bossElement2Mass(bossPdbAtom2Element(num2typ2symb[i][0]))) for i in range(len(Qs))]
    for i in range(len(Mass)):
        rtf.write('%s' % Mass[i])
    rtf.write('\nRESIdue %5s\n' % (resid))
    rtf.write('\nGROUP\n')
    rtf.write('\n{ atomName  atomType  Charge } \n')
    for i in range(len(Qs)):
        rtf.write('ATOM %6s TYPE= %6s CHARGE= %8s END\n' % (num2typ2symb[i][0], bossPdbAtom2Element(
            num2typ2symb[i][0]) + num2typ2symb[i][1][-3:], Qs[i][1]))
    rtf.write('\n{ Bonds: atomName1  atomName2 } \n')
    for (x, y) in zip(bnd_df.cl1, bnd_df.cl2):
        rtf.write('BOND %s %s \n' % (num2typ2symb[x][0], num2typ2symb[y][0]))
    rtf.write('\n{ Angles: atomName1 atomName2 atomName3} \n')
    for a in angs: rtf.write('ANGLe %s \n'%a) 
    rtf.write('\n{ Proper Dihedrals: aName1 aName2 aName3 aName4 } \n')
    for i in props:
        rtf.write('DIHEdral %s \n' % (i.replace("-", " ")))
    rtf.write('\n{ Improper Dihedrals: aName1 aName2 aName3 aName4 }\n')
    for i in imps:
        rtf.write('IMPRoper %s \n' % (i.replace("-", " ")))
    rtf.write('\nEND {RESIdue UNK}\n')
    rtf.write('\nset echo=true end\n')
    rtf.close()
    return None


def Boss2CharmmPRM(resid, num2typ2symb, Qs, bnd_df, ang_df, tor_df):
    #### COLLECTING NONBONDING PART #######
    prm = open(resid + '.param', 'w+')
    prm.write('Remarks LigParGen generated XPLOR-TOP file for Bonvin group (by Leela Dodda)\n')
    prm.write('\nset echo=false end \n')
    #    bnd_df = bnd_df.drop_duplicates(['TIJ'])
    prm.write('\n{ Bonds: atomType1 atomType2 kb r0 } \n')
    for i in bnd_df.index:
        prm.write('BOND %6s %6s %8.1f %8.4f \n' % (num2typ2symb[bnd_df.cl1[i]][
            2], num2typ2symb[bnd_df.cl2[i]][2], bnd_df.KIJ[i], bnd_df.RIJ[i]))
    #    ang_df = ang_df.drop_duplicates(['TY', 'R', 'K'])
    prm.write('\n{ Angles: aType1 aType2 aType3 kt t0 }\n')
    for i in ang_df.index:
        prm.write('ANGLe %5s %5s %5s %8.1f %8.2f \n' % (num2typ2symb[ang_df.cl1[i]][2], num2typ2symb[
            ang_df.cl2[i]][2], num2typ2symb[ang_df.cl3[i]][2], ang_df.K[i], ang_df.R[i]))
    prm.write('\n{ Proper Dihedrals: aType1 aType2 aType3 aType4 kt period phase } \n')
    if len(tor_df.index) > 0:
        tor_df = tor_df.drop_duplicates(['NAME', 'TY'])
    pro_df = tor_df[tor_df.TY == 'Proper']
    for i in list(pro_df.index):
        ndf = pro_df.ix[i]
        pro_out = retDihed(ndf.to_dict())
        for i in range(4):
            prm.write('%s' % pro_out[i])
    prm.write(
        'X    X    X    X    0.00000 1 0.000000 ! WILD CARD FOR MISSING TORSION PARAMETERS\n')
    prm.write('\n{ Improper Dihedrals: aType1 aType2 aType3 aType4 kt period phase }\n')
    imp_df = tor_df[tor_df.TY == 'Improper']
    for i in list(imp_df.index):
        ndf = tor_df.ix[i]
        imp_out = retDihedImp(ndf.to_dict())
        for i in range(len(imp_out)):
            prm.write('%s' % imp_out[i])
    prm.write(
        'IMPRoper X    X    X    X    0.00000 1 0.000000 ! WILD CARD FOR MISSING IMPROPER PARAMETERS \n')
    prm.write(
        '\n{ Nonbonded: Type Emin sigma; (1-4): Emin/2 sigma }\n')
    Qlines = ['NONBonded %5s %11.6f %11.6f %11.6f %11.6f \n' %
              (num2typ2symb[i][2], float(Qs[i][3]), float(Qs[i][2]), float(Qs[i][3]) * 0.50,float(Qs[i][2])) for i in range(len(Qs))]
    #    uniqQs = list(set(Qlines))
    for i in range(len(Qlines)):
        prm.write('%s' % Qlines[i])
    prm.write('\nset echo=true end \n')
    prm.close()
    return None


def Boss2CharmmTorsion(bnd_df, num2opls, st_no, molecule_data, num2typ2symb):
    #    print num2opls
    dhd = []
    for line in molecule_data.MolData['TORSIONS']:
        dt = [float(l) for l in line]
        dhd.append(dt)
    dhd = np.array(dhd)
    dhd = dhd  # kcal to kj conversion
    dhd = dhd / 2.0  # Komm = Vopls/2
    dhd_df = pd.DataFrame(dhd, columns=['V1', 'V2', 'V3', 'V4'])
    ats = []
    for line in molecule_data.MolData['ATOMS'][3:]:
        dt = [line.split()[0], line.split()[4],
              line.split()[6], line.split()[8]]
        dt = [int(d) for d in dt]
        ats.append(dt)
    for line in molecule_data.MolData['ADD_DIHED']:
        dt = [int(l) for l in line]
        ats.append(dt)
    assert len(ats) == len(
        dhd), 'Number of Dihedral angles in Zmatrix and Out file dont match'
    ats = np.array(ats) - st_no
    for i in range(len(ats)):
        for j in range(len(ats[0])):
            if ats[i][j] < 0:
                ats[i][j] = 0
    at_df = pd.DataFrame(ats, columns=['I', 'J', 'K', 'L'])
    final_df = pd.concat([dhd_df, at_df], axis=1, join_axes=[at_df.index])
    bndlist = list(bnd_df.UR) + (list(bnd_df.UR))
    final_df['TY'] = ['Proper' if ucomb(list([final_df.I[n], final_df.J[n], final_df.K[
        n], final_df.L[n]]), bndlist) == 3 else 'Improper' for n in range(len(final_df.I))]
#    final_df['SumV'] = np.abs(
#        final_df.V1) + np.abs(final_df.V2) + np.abs(final_df.V3) + np.abs(final_df.V4)
    #    final_df = final_df[final_df['SumV'] != 0.00]
    final_df['TI'] = [num2typ2symb[j][2] for j in final_df.I]
    final_df['TJ'] = [num2typ2symb[j][2] for j in final_df.J]
    final_df['TK'] = [num2typ2symb[j][2] for j in final_df.K]
    final_df['TL'] = [num2typ2symb[j][2] for j in final_df.L]
    final_df['SYMB'] = ['   '.join([num2typ2symb[final_df.I[i]][0], num2typ2symb[final_df.J[i]][
        0], num2typ2symb[final_df.K[i]][0], num2typ2symb[final_df.L[i]][0]]) for i in final_df.index]
    if len(final_df.index) > 0:
        final_df['NAME'] = final_df.TI + '-' + final_df.TJ + \
            '-' + final_df.TK + '-' + final_df.TL
    return final_df


def boss2CharmmBond(molecule_data, st_no):
    bdat = molecule_data.MolData['BONDS']
    bdat['cl1'] = [x - st_no if not x - st_no < 0 else 0 for x in bdat['cl1']]
    bdat['cl2'] = [x - st_no if not x - st_no < 0 else 0 for x in bdat['cl2']]
    bnd_df = pd.DataFrame(bdat)
    bnd_df['UF'] = ((bnd_df.cl1 + bnd_df.cl2) *
                    (bnd_df.cl1 + bnd_df.cl2 + 1) * 0.5) + bnd_df.cl2
    bnd_df['UR'] = ((bnd_df.cl1 + bnd_df.cl2) *
                    (bnd_df.cl1 + bnd_df.cl2 + 1) * 0.5) + bnd_df.cl1
#    bnd_df.to_csv('bos_bonds.csv', index=False)
    hb_df = bnd_df.drop(['cl1', 'cl2', 'UF', 'UR'], 1)
    hb_df = hb_df.drop_duplicates()
    return bnd_df


def boss2CharmmAngle(anglefile, num2opls, st_no,num2typ2symb):
    adat = anglefile
    adat['cl1'] = [x - st_no if not x - st_no < 0 else 0 for x in adat['cl1']]
    adat['cl2'] = [x - st_no if not x - st_no < 0 else 0 for x in adat['cl2']]
    adat['cl3'] = [x - st_no if not x - st_no < 0 else 0 for x in adat['cl3']]
    ang_df = pd.DataFrame(adat)
    ang_df = ang_df[ang_df.K > 0]
#    ang_df.to_csv('bos_angles.csv', index=False)
    ang_df['TY'] = np.array([num2opls[i] + '-' + num2opls[j] + '-' + num2opls[k]
                             for i, j, k in zip(ang_df.cl1, ang_df.cl2, ang_df.cl3)])
    ang_df['TI']=[num2typ2symb[ang_df.cl1[i]][2] for i in ang_df.index]
    ang_df['TJ']=[num2typ2symb[ang_df.cl2[i]][2] for i in ang_df.index]
    ang_df['TK']=[num2typ2symb[ang_df.cl3[i]][2] for i in ang_df.index]
    ang_df['TY'] = np.array([i + '  ' + j + '  ' + k
                             for i, j, k in zip(ang_df.TI, ang_df.TJ, ang_df.TK)])
    return ang_df


def bossData(molecule_data):
    ats_file = molecule_data.MolData['ATOMS']
    types = []
    for i in enumerate(ats_file):
        types.append([i[1].split()[1], 'opls_' + i[1].split()[2]])
    st_no = 3
    Qs = molecule_data.MolData['Q_LJ']
    assert len(Qs) == len(types), 'Please check the at_info and Q_LJ_dat files'
    num2opls = {}
    for i in range(0, len(types)):
        num2opls[i] = Qs[i][0]
    num2typ2symb = {i: types[i] for i in range(len(Qs))}
    for i in range(len(Qs)):
        num2typ2symb[i].append(bossPdbAtom2Element(
            num2typ2symb[i][0]) + num2typ2symb[i][1][-3:])
        num2typ2symb[i].append(bossPdbAtom2Element(num2typ2symb[i][0]))
        num2typ2symb[i].append(bossElement2Mass(num2typ2symb[i][3]))
        num2typ2symb[i].append(Qs[i][0])
    return (types, Qs, num2opls, st_no, num2typ2symb)


def Boss2Charmm(resid, molecule_data):
    types, Qs, num2opls, st_no, num2typ2symb = bossData(molecule_data)
    bnd_df = boss2CharmmBond(molecule_data, st_no)
    ang_df = boss2CharmmAngle(molecule_data.MolData['ANGLES'], num2opls, st_no,num2typ2symb)
    tor_df = Boss2CharmmTorsion(bnd_df, num2opls, st_no,
                                molecule_data, num2typ2symb)
    Boss2CharmmRTF(num2typ2symb, Qs, resid, bnd_df, list(ang_df['TY']), list(
        tor_df[tor_df.TY == 'Proper']['SYMB']),list(tor_df[tor_df.TY == 'Improper']['SYMB']))
    Boss2CharmmPRM(resid, num2typ2symb, Qs, bnd_df, ang_df, tor_df)
    return None


def mainBOSS2XPLOR(resid, clu=False):
    mol = pickle.load(open(resid + ".p", "rb"))
    Boss2Charmm(resid, mol)
    return None



"""
SCRIPT TO CONVERT WRITE OPENMM PDB AND XML FILES 
FROM BOSS ZMATRIX
Created on Mon Feb 15 15:40:05 2016
@author: Lela S. Dodda leela.dodda@yale.edu
@author: William L. Jorgensen Lab 

Usage: python OPM_Routines.py -z phenol.z -r PHN
REQUIREMENTS:
BOSS (need to set BOSSdir in bashrc and cshrc)
Preferably Anaconda python with following modules
pandas 
argparse
numpy
"""

from collections import OrderedDict
#from BOSSReader import *
from LigParGen.BOSSReader import Refine_PDB_file,get_coos_from_pdb
from LigParGen.BOSSReader import ucomb,bossPdbAtom2Element,bossElement2Mass,tor_cent
import pickle
import pandas as pd
import numpy as np


def printDihed(tdat):
    return (
        '<%s class1=\"%s\" class2=\"%s\" class3=\"%s\" class4=\"%s\" k1=\"%6.6f\" k2=\"%6.6f\" k3=\"%6.6f\" k4=\"%6.6f\" periodicity1=\"1\" periodicity2=\"2\" periodicity3=\"3\" periodicity4=\"4\" phase1=\"0.00\" phase2=\"3.141592653589793\" phase3=\"0.00\" phase4=\"3.141592653589793\"/>' % (
            tdat[-1], tdat[0], tdat[1], tdat[2], tdat[3], tdat[4], tdat[5], tdat[6], tdat[7]))


def boss2opmAtom(num2typ2symb, xmlf):
    xmlf.write('<AtomTypes>\n')
    pr_opmAtom = ['<Type name=\"%s\" class=\"%s\" element=\"%s\" mass=\"%6.6f\" />\n' %
                  (num2typ2symb[n][1], num2typ2symb[n][2], num2typ2symb[n][3], num2typ2symb[n][4]) for n in
                  num2typ2symb.keys()]
    for n in list(set(pr_opmAtom)):
        xmlf.write('%s' % n)
    xmlf.write('</AtomTypes>\n')
    return None


def boss2opmTorsion(bnd_df, num2opls, st_no, molecule_data, xmlf):
    dhd = []
    for line in molecule_data.MolData['TORSIONS']:
        dt = [float(l) for l in line]
        dhd.append(dt)
    dhd = np.array(dhd)
    dhd = dhd * 4.184  # kcal to kj conversion
    dhd = dhd / 2.0  # Komm = Vopls/2
    dhd_df = pd.DataFrame(dhd, columns=['V1', 'V2', 'V3', 'V4'])
    ats = []
    for line in molecule_data.MolData['ATOMS'][3:]:
        dt = [line.split()[0], line.split()[4],
              line.split()[6], line.split()[8]]
        dt = [int(d) for d in dt]
        ats.append(dt)
    for line in molecule_data.MolData['ADD_DIHED']:
        dt = [int(l) for l in line]
        ats.append(dt)
    assert len(ats) == len(
        dhd), 'Number of Dihedral angles in Zmatrix and Out file dont match'
    ats = np.array(ats) - st_no
    for i in range(len(ats)):
        for j in range(len(ats[0])):
            if ats[i][j] < 0:
                ats[i][j] = 0
    at_df = pd.DataFrame(ats, columns=['I', 'J', 'K', 'L'])
    final_df = pd.concat([dhd_df, at_df], axis=1, join_axes=[at_df.index])
    bndlist = list(bnd_df.UR) + (list(bnd_df.UR))
    final_df['TY'] = ['Proper' if ucomb(list([final_df.I[n], final_df.J[n], final_df.K[
        n], final_df.L[n]]), bndlist) == 3 else 'Improper' for n in range(len(final_df.I))]
    final_df['SumV'] = np.abs(
        final_df.V1) + np.abs(final_df.V2) + np.abs(final_df.V3) + np.abs(final_df.V4)
    if len(final_df.index) >= 1:
        final_df['TI'] = [num2opls[j] for j in final_df.I]
        final_df['TJ'] = [num2opls[j] for j in final_df.J]
        final_df['TK'] = [num2opls[j] for j in final_df.K]
        final_df['TL'] = [num2opls[j] for j in final_df.L]
        final_df['NAME'] = final_df.TI + '-' + final_df.TJ + \
            '-' + final_df.TK + '-' + final_df.TL
        final_df = final_df.sort_values(['NAME'])
        final_df.to_csv('bos_torsions.csv', index=False)
        tor_bos = final_df.drop(
            ['I', 'J', 'K', 'L', 'TI', 'TJ', 'TK', 'TL'], 1)
        tor_bos = tor_bos.drop_duplicates()
        df = final_df.ix[tor_bos.index][['TI', 'TJ', 'TK', 'TL',
                                         'V1', 'V2', 'V3', 'V4', 'TY', 'I', 'J', 'K', 'L']]
        torlist = []
        for row in df[df.TY == 'Proper'].iterrows():
            index, dat = row
            data = dat.tolist()
            torlist.append(printDihed(data[0:-4]))
        for row in df[df.TY == 'Improper'].iterrows():
            index, dat = row
            ndata = tor_cent([dat.I, dat.J, dat.K, dat.L], bndlist)
            sdata = [num2opls[j] for j in ndata]
            vdata = [dat.V1, dat.V2, dat.V3, dat.V4, dat.TY]
            if len(set([dat.TI, dat.TJ, dat.TK, dat.TL])) > 3:
                torlist.append(printDihed(sdata + vdata))
#        fin_list = list(set(torlist))
        xmlf.write('<PeriodicTorsionForce>\n')
        for npt in torlist:
            xmlf.write("%s\n" % npt)
        xmlf.write('</PeriodicTorsionForce>\n')
    return None


def boss2opmBond(num2opls, molecule_data, st_no, xmlf):
    bdat = molecule_data.MolData['BONDS']
    bdat['cl1'] = [x - st_no if not x - st_no < 0 else 0 for x in bdat['cl1']]
    bdat['cl2'] = [x - st_no if not x - st_no < 0 else 0 for x in bdat['cl2']]
    bnd_df = pd.DataFrame(bdat)
    bnd_df['T1'] = [num2opls[x] for x in bnd_df.cl1]
    bnd_df['T2'] = [num2opls[x] for x in bnd_df.cl2]
    bnd_df['KIJ'] = bnd_df['KIJ'] * 836.80
    bnd_df['RIJ'] = bnd_df['RIJ'] * 0.10
    bnd_df['UF'] = ((bnd_df.cl1 + bnd_df.cl2) *
                    (bnd_df.cl1 + bnd_df.cl2 + 1) * 0.5) + bnd_df.cl2
    bnd_df['UR'] = ((bnd_df.cl1 + bnd_df.cl2) *
                    (bnd_df.cl1 + bnd_df.cl2 + 1) * 0.5) + bnd_df.cl1
    connects = []
    for (ai, aj) in zip(bnd_df.cl1, bnd_df.cl2):
        xmlf.write('<Bond from=\"%d\" to=\"%d\"/>\n' % (aj, ai))
        connects.append("CONECT{:>5}{:>5}".format(aj + 1, ai + 1))
    xmlf.write('</Residue>\n')
    xmlf.write('</Residues>\n')
    full_bnd = bnd_df.copy()
#    bnd_df = bnd_df.drop_duplicates(['TIJ'])
    xmlf.write('<HarmonicBondForce>\n')
#    for bd in range(0, len(bnd_df.TIJ)):
#        a1, a2 = list(bnd_df['TIJ'])[bd].split('-')
#        a1 = a1.strip()
#        a2 = a2.strip()
#        xmlf.write('<Bond class1=\"%s\" class2=\"%s\" length=\"%6.6f\" k=\"%6.6f\"/>\n' %
#                   (a1, a2, list(bnd_df.RIJ)[bd], list(bnd_df.KIJ)[bd]))
    for i in bnd_df.index:
        xmlf.write('<Bond class1=\"%s\" class2=\"%s\" length=\"%6.6f\" k=\"%6.6f\"/>\n' %
                   (bnd_df.ix[i]['T1'], bnd_df.ix[i]['T2'], bnd_df.ix[i]['RIJ'], bnd_df.ix[i]['KIJ']))
    xmlf.write('</HarmonicBondForce>\n')
    return full_bnd, connects


def boss2opmAngle(anglefile, num2opls, st_no, xmlf):
    adat = anglefile
    adat['cl1'] = [x - st_no if not x - st_no < 0 else 0 for x in adat['cl1']]
    adat['cl2'] = [x - st_no if not x - st_no < 0 else 0 for x in adat['cl2']]
    adat['cl3'] = [x - st_no if not x - st_no < 0 else 0 for x in adat['cl3']]
    ang_df = pd.DataFrame(adat)
    ang_df = ang_df[ang_df.K > 0]
    ang_df['K'] = 8.3680 * ang_df['K']
    ang_df['R'] = (np.pi / 180.0) * ang_df['R']
#    ang_df.to_csv('bos_angles.csv', index=False)
    ang_df['TY'] = np.array([num2opls[i] + '-' + num2opls[j] + '-' + num2opls[k]
                             for i, j, k in zip(ang_df.cl1, ang_df.cl2, ang_df.cl3)])
    ang_df = ang_df.drop_duplicates(['TY'])
    xmlf.write('<HarmonicAngleForce>\n')
    for i in range(0, len(ang_df.index)):
        xmlf.write('<Angle class1=\"%s\" class2=\"%s\" class3=\"%s\" angle=\"%6.6f\" k=\"%6.6f\"/>\n' % (
            list(ang_df.TY)[i].split('-')[
                0], list(ang_df.TY)[i].split('-')[1], list(ang_df['TY'])[i].split('-')[2], list(ang_df['R'])[i],
            list(ang_df['K'])[i]))
    xmlf.write('</HarmonicAngleForce>\n')
    return None


def bossData(molecule_data):
    ats_file = molecule_data.MolData['ATOMS']
    types = []
    for i in enumerate(ats_file):
        types.append([i[1].split()[1], 'opls_' + i[1].split()[2]])
    st_no = 3
    Qs = molecule_data.MolData['Q_LJ']
    assert len(Qs) == len(types), 'Please check the at_info and Q_LJ_dat files'
#    num2opls = {}
#    for i in range(0, len(types)):
#        num2opls[i] = Qs[i][0]
    num2typ2symb = {i: types[i] for i in range(len(Qs))}
    for i in range(len(Qs)):
        num2typ2symb[i].append(bossPdbAtom2Element(
            num2typ2symb[i][0]) + num2typ2symb[i][1][-3:])
#        print(num2typ2symb[i],bossElement2Mass(num2typ2symb[i][0]))
        num2typ2symb[i].append(bossPdbAtom2Element(num2typ2symb[i][0]))
        num2typ2symb[i].append(bossElement2Mass(num2typ2symb[i][3]))
        num2typ2symb[i].append(Qs[i][0])
    num2opls = {}
    for i in num2typ2symb.keys():
        num2opls[i] = num2typ2symb[i][2]
#    return (types, Qs, num2opls, st_no, num2typ2symb)
    num2pqrtype = OrderedDict(num2typ2symb)
    for i in range(len(Qs)):
        num2pqrtype[i].append(Qs[i][1])
        num2pqrtype[i].append(Qs[i][2])
    return (types, Qs, num2opls, st_no, num2typ2symb, num2pqrtype)


def pdb_prep(atoms, coos, resid, connects):
    print(resid + '.pdb')
    opdb = open(resid + '.pdb', 'w+')
    opdb.write('REMARK LIGPARGEN GENERATED PDB FILE\n')
    num = 0
    for (i, j) in zip(atoms, coos):
        num += 1
        opdb.write('%-6s%5d %4s %3s  %4d    %8.3f%8.3f%8.3f\n' %
                   ('ATOM', num, i, resid, 1, j[0], j[1], j[2]))
    opdb.write('TER \n')
    for i in range(len(connects)):
        opdb.write('%s \n' % connects[i])
    opdb.write('END')
    opdb.close()
    return None


def pqr_prep(atoms, coos, resid, connects, num2pqrtype):
    at2pqrtype = {num2pqrtype[i][0]: num2pqrtype[i][-2:]
                  for i in num2pqrtype.keys()}
#    print(at2pqrtype)
    opdb = open(resid + '.pqr', 'w+')
    opdb.write('REMARK LIGPARGEN GENERATED PQR FILE\n')
    num = 0
    for (i, j) in zip(atoms, coos):
        num += 1
        opdb.write('%-6s%5d %4s %3s  %4d    %8.3f%8.3f%8.3f%8.4f%7.4f\n' %
                   ('ATOM', num, i, resid, 1, j[0], j[1], j[2], float(at2pqrtype[i][0]), float(at2pqrtype[i][1]) * 0.561231))
    opdb.write('TER \n')
    for i in range(len(connects)):
        opdb.write('%s \n' % connects[i])
    opdb.write('END')
    opdb.close()
    return None


def boss2opm(resid, molecule_data, pdb_file):
    xmlf = open(resid + '.xml', 'w+')
    types, Qs, num2opls, st_no, num2typ2symb, num2pqrtype = bossData(
        molecule_data)
    #    print(num2typ2symb)
    #### COLLECTING NONBONDING PART #######
    nb_part = []
    res_at = []
    for i in range(len(types)):
        nb_part.append('<Atom type=\"%s\" charge=\"%3.6f\" sigma=\"%3.6f\" epsilon=\"%3.6f\" />\n' %
                       (types[i][1], float(Qs[i][1]), float(Qs[i][2]) / 10.0, float(Qs[i][3]) * 4.184))
        res_at.append('<Atom name=\"%s\" type=\"%s\" />\n' %
                      (types[i][0], types[i][1]))
    xmlf.write('<ForceField>\n')
    boss2opmAtom(num2typ2symb, xmlf)
    #### PRINTING ATOM PART #######
    xmlf.write('<Residues>\n')
    xmlf.write('<Residue name=\"%s\">\n' % resid)
    for at in res_at:
        xmlf.write("%s" % at)
    bnd_df, connects = boss2opmBond(num2opls, molecule_data, st_no, xmlf)
    # PRINTING ANGLES AND TORSIONS
    boss2opmAngle(molecule_data.MolData['ANGLES'], num2opls, st_no, xmlf)
    boss2opmTorsion(bnd_df, num2opls, st_no, molecule_data, xmlf)
    #### PRINTING NONBONDING PART #######
    nnb_part = list(set(nb_part))
    xmlf.write('<NonbondedForce coulomb14scale="0.5" lj14scale="0.5">\n')
    for nb in nnb_part:
        xmlf.write("%s" % nb)
    xmlf.write('</NonbondedForce>\n')
    xmlf.write('</ForceField>\n')
    xmlf.close()
#    pdb_prep(molecule_data.MolData['PDB'], connects, resid)
    pdblines = Refine_PDB_file(pdb_file)
    atoms, coos = get_coos_from_pdb(pdblines)
    pdb_prep(atoms, coos, resid, connects)
    pqr_prep(atoms, coos, resid, connects, num2pqrtype)
    return None


def mainBOSS2OPM(resid, clu):
    mol = pickle.load(open(resid + ".p", "rb"))
    if clu:
        pdb_file = '/tmp/clu.pdb'
    else:
        pdb_file = '/tmp/plt.pdb'
    boss2opm(resid, mol, pdb_file)
    return None



import numpy as np


def new_func(linex, match):
    out = 0
    for word in linex.split():
        if(word == match):
            out = out + 1
    return out


def read_coords(data):
    cmatrix = []
    ta = []
    tb = []
    for i in range(0, len(data)):
        cmatrix.append(data[i].split())
    ta = [int(cmatrix[i][2]) for i in range(0, len(data))]
    tb = [int(cmatrix[i][3]) for i in range(0, len(data))]
    ta = np.array(ta)
    maxa = ta.max()
    tb = np.array(tb)
    numi = 1
    for i in range(0, len(tb)):
        if(tb[i] > 1):
            tb[i] = maxa + numi
            numi = numi + 1
    for i in range(0, len(data)):
        cmatrix[i][3] = str(tb[i])
    outdat = []
    new_coord = ''
    for i in range(0, len(data)):
        new_coord = '{:>4s} {:<3s} {:>4s} {:>4s}'.format(
            cmatrix[i][0],
            cmatrix[i][1],
            cmatrix[i][2],
            cmatrix[i][3])
        new_coord = new_coord + '{:>5s}{:>12s}{:>4s}{:>12s}'.format(
            cmatrix[i][4],
            cmatrix[i][5],
            cmatrix[i][6],
            cmatrix[i][7])
        new_coord = new_coord + \
            '{:>4s}{:>12s}{:>9s}'.format(
                cmatrix[i][8],
                cmatrix[i][9],
                cmatrix[i][10])
        outdat.append(new_coord)
    tb = tb[tb > 0]  # IMPORTANT TO AVOID THE -1 and 0 IN FINAL ATOM TYPE
    ta = ta[ta > 0]  # IMPORTANT TO AVOID THE -1 and 0 IN FINAL ATOM TYPE
    return outdat, tb, ta


def read_files(infile):
    nline = 0
    cline = 0
    oline = 0
    data = []
    for line in infile:
        if line.rstrip():
            data.append(line)
            if("Non-Bonded" in line):
                oline = nline
            elif("Variations" in line):
                cline = nline
            nline += 1
    return data, nline, cline, oline


def rel_nbd(data, tb, QBCC=None):
    if QBCC is None:
        QBCC = np.zeros(len(data), dtype=float)
    nmat = []
    nmat = [ndat.split() for ndat in data]
    ondat = []
    for i in range(0, len(data)):
        nmat[i][0] = str(tb[i])
        nmat[i][3] = '%.6f' % QBCC[i]
        new_nb = '{:>4s}{:>3s} {:<3s} {:>9s} {:>9s} {:>9s}'.format(
            nmat[i][0],
            nmat[i][1],
            nmat[i][2],
            nmat[i][3],
            nmat[i][4],
            nmat[i][5])
        ondat.append(new_nb)
    return ondat


def fepZmatFromFile(filenme, QBCC=None):
    qfile = open(filenme)
    qdat, nl1, cl1, ol1 = read_files(qfile)
    cdat, tb, ta = read_coords(qdat[1:cl1])
    ndat = rel_nbd(qdat[ol1 + 1:], tb, QBCC)
    qdat[ol1] = qdat[ol1].replace(
        "AM1 CM1Ax1.14", "CM1Ax1.14TO1.14CM1A-BCC", 1)
    target = open(filenme[:-2] + '_fep.z', 'w')
    target.write(qdat[0])
    for i in range(0, len(cdat)):
        target.write(cdat[i] + '\n')
    for i in range(cl1, nl1):
        target.write(qdat[i])
    for i in range(0, len(ndat)):
        target.write(ndat[i] + '\n')
    target.close()
    return None


def fepZmatFromPkl(zmat_dat, filenme, QBCC=None):
    qdat, nl1, cl1, ol1 = read_files(zmat_dat)
    cdat, tb, ta = read_coords(qdat[1:cl1])
    ndat = rel_nbd(qdat[ol1 + 1:], tb, QBCC)
    qdat[ol1] = qdat[ol1].replace(
        "AM1 CM1Ax1.14", "CM1Ax1.14TO1.14CM1A-BCC", 1)
    target = open(filenme + '_fep.z', 'w')
    target.write(qdat[0])
    for i in range(0, len(cdat)):
        target.write(cdat[i] + '\n')
    for i in range(cl1, nl1):
        target.write(qdat[i])
    for i in range(0, len(ndat)):
        target.write(ndat[i] + '\n')
    target.close()
    return None


def BCC_file2zmat(zmat, QBCC, oname):
    qfile = open(zmat, 'r+')
    qdat, nl1, cl1, ol1 = read_files(qfile)
    cdat, tb, ta = read_coords(qdat[1:cl1])
    ndat = rel_nbd(qdat[ol1 + 1:], ta, QBCC)
    qdat[ol1] = qdat[ol1].replace("AM1 CM1Ax1.14", "1.14CM1A-BCC", 1)
    qfile.close()
    # oname=zmat[:-2]
    target = open(oname, 'w+')
    print('WRITING TO ' + oname)
#    target.write(qdat[0])
#    for i in range(0, len(cdat)):
#        target.write(cdat[i] + '\n')
    for i in range(0, ol1 + 1):
        target.write(qdat[i])
    for i in range(0, len(ndat)):
        target.write(ndat[i] + '\n')
    target.close()
    return None


def BCC_pkl2zmat(zmat_dat, QBCC, zname):
    qdat, nl1, cl1, ol1 = read_files(zmat_dat)
    cdat, tb, ta = read_coords(qdat[1:cl1])
    ndat = rel_nbd(qdat[ol1 + 1:], ta, QBCC)
    qdat[ol1] = qdat[ol1].replace("AM1 CM1Ax1.14", "1.14CM1A-BCC", 1)
    target = open(zname + '_BCC.z', 'w')
    for i in range(0, ol1 + 1):
        target.write(qdat[i])
    for i in range(0, len(ndat)):
        target.write(ndat[i] + '\n')
    target.close()
    return None


def bccZmat(zmat, QBCC, zmat_dat):
    if zmat_dat is not None:
        fepZmatFromPkl(zmat_dat, zmat, QBCC)
    elif zmat is not None:
        BCC_file2zmat(zmat, QBCC)
    return None


def fepZmat(zmat, zmat_dat, QBCC):
    if zmat_dat is not None:
        fepZmatFromPkl(zmat_dat,  zmat, QBCC)
    elif zmat is not None:
        fepZmatFromFile(zmat, QBCC)
    return None


def Par_BCC_file2zmat(filenme, QBCC, oname):
    qfile = open(filenme)
    qdat, nl1, cl1, ol1 = read_files(qfile)
    cdat, tb, ta = read_coords(qdat[1:cl1])
    ndat = rel_nbd(qdat[ol1 + 1:], tb, QBCC)
    qdat[ol1] = qdat[ol1].replace(
        "AM1 CM1Ax1.14", "CM1Ax1.14TO1.14CM1A-BCC", 1)
    print('IAM HERE writing to %s' % oname)
    target = open(oname, 'w+')
    target.write(qdat[0])
    for i in range(0, len(cdat)):
        target.write(cdat[i] + '\n')
    for i in range(cl1, nl1):
        target.write(qdat[i])
    for i in range(0, len(ndat)):
        target.write(ndat[i] + '\n')
    target.close()
    return None



from __future__ import print_function

import os
import numpy as np
from LigParGen import mol_boss
import pandas as pd
from collections import OrderedDict

from LigParGen.fepzmat import BCC_file2zmat
from LigParGen.CreatZmat import GenMolRep 

def VerifyMolandSave(mol,charge,resname):
    if mol is not None: 
        import pickle
        assert (mol.MolData['TotalQ']['Reference-Solute'] ==
                charge), "PROPOSED CHARGE IS NOT POSSIBLE: SOLUTE MAY BE AN OPEN SHELL"
        pickle.dump(mol, open(resname + ".p", "wb"))
    else: 
        print('Problem Detected Molecule Object Not created')
    return None

def LinCheck(fname):
    imp_dat = 0
    zlines  = open(fname,'r').readlines()
    for l in range(len(zlines)):
        if 'Geometry Variations follow ' in zlines[l]: imp_dat = l
    Atypes = []
    for l in zlines[1:imp_dat]:Atypes.append(l.split()[2])
    Atypes = np.array(Atypes,dtype=np.int)
    Atypes = Atypes[Atypes<0]
    Check =False
    if len(Atypes)>2: Check = True
    return Check

def mod_add_diheds(line):
    adihed = [int(l) for l in line.split()[0:4]]+[-1,-1]
    return(adihed)

def fix_add_dihed(zmat_name):
    flines = open('%s.z'%zmat_name,'r').readlines()
    imp_lines=[]
    for l in range(len(flines)):
        if 'Additional Dihedrals follow' in flines[l]:
            imp_lines.append(l+1)
        elif 'Domain Definitions follow' in flines[l]:
            imp_lines.append(l)
    ofile = open('%s_fixed.z'%zmat_name,'w+')
    for line in flines[0:imp_lines[0]]: ofile.write('%s\n'%(line.rstrip()))
    for line in flines[imp_lines[0]:imp_lines[1]]:
        m_ad = mod_add_diheds(line)
        ofile.write('%4d%4d%4d%4d%4d%4d\n'%(m_ad[0],m_ad[1],m_ad[2],m_ad[3],m_ad[4],m_ad[5]))
    for line in flines[imp_lines[1]:]: ofile.write('%s\n'%line.rstrip())
    ofile.close()
    return(None)

def CheckForHs(atoms):
    atype = [line.split()[1][0] for line in atoms]
    ans = False
    if ('H' in atype): ans = True 
    return ans 

def bcc_db(file):
    df = pd.read_csv(file)
    db = OrderedDict()
    for name, value in zip(df.BTYPE, df.BCC):
        db[name] = value
    return db


def Refine_PDB_file(fname):
    flines = open(fname, 'r+').readlines()
    pdb_lines = []
    for line in flines:
        if ('ATOM' in line) or ('HETATM' in line):
            line = line.rstrip()
            line = line.lstrip()
            if not 'DUM' in line:
                pdb_lines.append(line)
    return pdb_lines


def get_coos_from_pdb(pdb_dat):
    atoms = []
    coos = []
    for line in pdb_dat:
        atom = line.split()[2]
        x, y, z = line[28:56].split()
        atoms.append(atom)
        coos.append([float(x), float(y), float(z)])
    return (atoms, coos)

def pairing_func(a, b):
    ans = (a + b) * (a + b + 1) * 0.5
    if a > b:
        ans = ans + a
        pans = '%6d%6d' % (b, a)
    else:
        ans = ans + b
        pans = '%6d%6d' % (a, b)
    return (int(ans), pans)


def ucomb(vec, blist):
    res = 0
    for a in vec:
        vec.remove(a)
        for b in vec:
            ans = (a + b) * (a + b + 1) * 0.5
            if (ans + a in blist) or (ans + b in blist):
                res = res + 1
    return res


def tor_cent(vec, blist):
    db = {}
    for a in vec:
        na = 0
        for b in vec:
            ans = (a + b) * (a + b + 1) * 0.5
            if (ans + a in blist) or (ans + b in blist):
                na += 1
        db[a] = na
#    print(db)
    new_vec = list(sorted(db, key=db.__getitem__, reverse=True))
    return (new_vec)


def bossPdbAtom2Element(attype):
    elem = ''.join([i for i in attype[:-1] if not i.isdigit()])
    return elem


def bossElement2Mass(elem):
    symb2mass = {
        'H': 1.008,
        'F': 18.998403163,
        'Cl': 35.45,
        'Br': 79.904,
        'I': 126.90447,
        'O': 15.999,
        'S': 32.06,
        'N': 14.007,
        'P': 30.973761998,
        'C': 12.011,
        'Si': 28.085,
        'Na': 22.98976928,
        'SOD': 22.98976928,
        'K': 39.0983,
        'Mg': 24.305,
        'Ca': 40.078,
        'Mn': 54.938044,
        'Fe': 55.845,
        'Co': 58.933194,
        'Ni': 58.6934,
        'Cu': 63.546,
        'Zn': 65.38, }
    try:
        res = symb2mass[elem]
    except NameError:
        print("Mass for atom %s is not available \n add it to symb2mass dictionary")
    return res


def Refine_file(fname):
    flines = open(fname, 'r+')
    lines = []
    for line in flines:
        if line.rstrip():
            line = line.rstrip()
            line = line.lstrip()
            lines.append(line)
    flines.close()
    return lines


class BOSSReader(object):

    def __init__(self, zmatrix, optim, charge=0, lbcc=False):
        self.zmat = zmatrix
        self.impDat = {}
        self.MolData = {}
        self.refine_data(optim, charge, lbcc)

    def Get_OPT(self, optim, charge):
        assert os.path.isfile(
            self.zmat), 'File named %10s does not exist' % self.zmat
        assert ('BOSSdir' in os.environ) and os.path.isfile((os.environ[
            'BOSSdir'] + '/scripts/xZCM1A')), 'Please Make sure $BOSSdir is defined \n xZCM1A and related files are in scripts directory of BOSS'
        execs = {
             2: os.environ['BOSSdir'] + '/scripts/xZCM1A+2 > /tmp/olog',
             1: os.environ['BOSSdir'] + '/scripts/xZCM1A+  > /tmp/olog',
             0: os.environ['BOSSdir'] + '/scripts/xZCM1A > /tmp/olog',
            -1: os.environ['BOSSdir'] + '/scripts/xZCM1A-  > /tmp/olog',
            -2: os.environ['BOSSdir'] + '/scripts/xZCM1A-2 > /tmp/olog',
        }
        #print('MOLECULE HAS A CHARGE of %d' % charge)
        if optim > 0:
            print('Optimization level requested %d' % optim)
            for opt_lev in range(optim):
                print('Performing Stage %d of Charge Generation'%(opt_lev+1)) 
                execfile = execs[charge]
                coma = execfile + ' ' + self.zmat[:-2]
                os.system(coma)
                os.system('cp sum %s' % (self.zmat))
                execfile = os.environ['BOSSdir'] + '/scripts/xOPT > /tmp/olog'
                coma = execfile + ' ' + self.zmat[:-2]
                os.system(coma)
                os.system('cd /tmp;/bin/cp sum %s' % (self.zmat))
        execfile = os.environ['BOSSdir'] + '/scripts/xSPM > /tmp/olog'
        coma = execfile + ' ' + self.zmat[:-2]
        os.system(coma)
        os.system('cd /tmp;/bin/cp sum %s' % (self.zmat))
        return (None)

    def get_addihed(self, data):
        add = []
        nadd = 0
        for line in data:
            if line[0].isdigit():
                add.append(line.split()[0:4])
                nadd = nadd + 1
        return (add)

    def get_atinfo(self, data):
        ats = []
        nat = 0
        for line in data:
            if line[0].isdigit() and float(line.split()[2]) > 1:
                ats.append(line)
                nat += 1
        return (ats)

    def get_charge(self, data):
        TotQ = {}
        for line in data[1:]:
            words = line.split()
            TotQ['-'.join(words[:-1])] = round(float(words[-1]), 3)
#        print(TotQ)
        return TotQ

    def get_tors(self, data):
        tors = []
        ntor = 0
        for line in data:
            if 'All Solutes' in line:
                tors.append(line.split()[4:8])
                for tor in line.split()[4:8]:
                    if abs(float(tor)) > 0.0:
                        ntor = ntor + 1
        return (tors)

    def get_QLJ(self, data):
        qlj = []
        nqlj = 0
        for line in data:
            if 'All Solutes' in line and line[0].isalpha():
                qlj.append([line.split()[0], line.split()[2],
                            line.split()[3], line.split()[4]])
                nqlj += 1
        return (qlj)

    def get_angs(self, data):
        angs = {'cl1': [], 'cl2': [], 'cl3': [], 'R': [], 'K': []}
        nang = 0
        for line in data:
            if line[0].isdigit() and float(line.split()[4]) > 0:
                word = line.split()
                angs['cl1'].append(int(word[0]))
                angs['cl2'].append(int(word[1]))
                angs['cl3'].append(int(word[2]))
                angs['R'].append(float(word[3]))
                angs['K'].append(float(word[4]))
                nang = nang + 1
            #        print 'Total No of Non-zero Angles in BOSS is %d' % (nang)
        return (angs)

    def get_XYZ(self, data):
        XYZ = {'at_num': [], 'X': [], 'Y': [], 'Z': [], 'at_symb': []}
        for line in data:
            if line[0].isdigit() and len(line.split()) == 5:
                word = line.split()
                XYZ['at_num'].append(int(word[0]))
                XYZ['X'].append(float(word[1]))
                XYZ['Y'].append(float(word[2]))
                XYZ['Z'].append(float(word[3]))
                XYZ['at_symb'].append(float(word[4]))
        return XYZ

    def get_pairs(self, data):
        data = data[1:]
        plnos = []
        for i in range(0, len(data)):
            if 'Atom' in data[i]:
                plnos.append(i)
        plnos.append(len(data))
        pair_dat = {i: ' '.join(data[plnos[i]:plnos[i + 1]])
                    for i in range(len(plnos) - 1)}
        for nu in range(len(plnos) - 1):
            pair_dat[nu] = list(pair_dat[nu][10:].split())
            pair_dat[nu] = np.array([int(a) - 2 for a in pair_dat[nu]])
        pairs = []
        for k in pair_dat.keys():
            for j in pair_dat[k]:
                pairs.append('%6d%6d%6d\n' % (k - 1, j, 1))
        return pairs

    def get_bonds(self, data):
        bnds = {'cl1': [], 'cl2': [], 'RIJ': [], 'KIJ': [], 'TIJ': []}
        nbnd = 0
        for line in data:
            if line[0].isdigit() and float(line.split()[3]) > 0:
                word = line.split()
                bnds['cl1'].append(int(word[0]))
                bnds['cl2'].append(int(word[1]))
                bnds['RIJ'].append(float(word[2]))
                bnds['KIJ'].append(float(word[3]))
                bnds['TIJ'].append(line[-5:])
                nbnd += 1
            #        print 'Total No of Non-zero Bonds in BOSS is %d' % (nbnd)
        return (bnds)

    def prep_lbcc(self, bond_data, qdata):
        db = bcc_db('/Users/Sriram/Dropbox/Paprica/NAR/Code/Debug/bonds.csv')
        bnd_df = pd.DataFrame(bond_data)
        bnd_df = bnd_df[['cl1', 'cl2']]
        bnd_df.columns = ['I', 'J']
        q_df = pd.DataFrame(columns=['TY', 'Q'])
        q_df.loc[0] = ['1', 0.000]
        q_df.loc[1] = ['2', 0.000]
        for i in range(len(qdata)):
            q_df.loc[i + 2] = [qdata[i][0], float(qdata[i][1])]
        bond, cha, QBC1 = mol_boss.new_mol_info(db, q_df, bnd_df)
        lbcc_qdat = []
        for i in range(len(qdata)):
            lbcc_qdat.append(
                [qdata[i][0], str(cha.QBCC.values[i]), qdata[i][2], qdata[i][3]])
        bond.to_csv('LBCC_BONDS.csv', index=False)
        cha.to_csv('LBCC_CHARGES.csv', index=False)
        print(bond)
        print(cha)
        return np.array(cha.QBCC), lbcc_qdat

    def cleanup(self):
        os.system('cd /tmp;/bin/rm sum log olog out plt.pdb')

    def get_ImpDat(self, optim, charge):
        self.Get_OPT(optim, charge)
        odat = Refine_file('/tmp/out')
        sdat = Refine_file('/tmp/sum')
        MolData = {}
        impDat = {}
        MolData['PDB'] = Refine_file('/tmp/plt.pdb')
        for nl in range(len(odat)):
            if 'Z-Matrix for Reference Solutes' in odat[nl]:
                impDat['ATMinit'] = nl
            elif 'Net Charge' in odat[nl]:
                impDat['TotalQ'] = nl
            elif 'OPLS Force Field Parameters' in odat[nl]:
                impDat['ATMfinal'] = nl
                impDat['NBDinit'] = nl
            elif 'Fourier Coefficients' in odat[nl]:
                impDat['TORinit'] = nl
                impDat['NBDfinal'] = nl
            elif 'Bond Stretching Parameters' in odat[nl]:
                impDat['TORfinal'] = nl
                impDat['BNDinit'] = nl
            elif 'Angle Bending Parameters' in odat[nl]:
                impDat['BNDfinal'] = nl
                impDat['ANGinit'] = nl
            elif 'Non-bonded Pairs List' in odat[nl]:
                impDat['ANGfinal'] = nl
                impDat['PAIRinit'] = nl
            elif 'Solute 0:   X          Y          Z' in odat[nl]:
                impDat['XYZinit'] = nl
            elif 'Atom I      Atom J      RIJ' in odat[nl]:
                impDat['XYZfinal'] = nl
            elif 'Checking' in odat[nl]:
                impDat['PAIRfinal'] = nl
#### THIS PART IS READ FROM SUM FILE ###
        for ml in range(len(sdat)):
            if 'Additional Dihedrals follow' in sdat[ml]:
                impDat['ADDinit'] = ml
            elif 'Domain Definitions follow' in sdat[ml]:
                impDat['ADDfinal'] = ml
#### THIS PART IS READ FROM SUM FILE ###
        MolData['ATOMS'] = self.get_atinfo(
            odat[impDat['ATMinit']:impDat['ATMfinal']])
        MolData['Q_LJ'] = self.get_QLJ(
            odat[impDat['NBDinit']:impDat['NBDfinal']])
        MolData['BONDS'] = self.get_bonds(
            odat[impDat['BNDinit']:impDat['BNDfinal']])
        MolData['ANGLES'] = self.get_angs(
            odat[impDat['ANGinit']:impDat['ANGfinal']])
        MolData['TORSIONS'] = self.get_tors(
            odat[impDat['TORinit']:impDat['TORfinal']])
        MolData['ADD_DIHED'] = self.get_addihed(
            sdat[impDat['ADDinit']:impDat['ADDfinal']])
        MolData['XYZ'] = self.get_XYZ(
            sdat[impDat['XYZinit']:impDat['XYZfinal']])
        MolData['PAIRS'] = self.get_pairs(
            odat[impDat['PAIRinit']:impDat['PAIRfinal']])
        MolData['TotalQ'] = self.get_charge(
            odat[impDat['TotalQ']:impDat['TotalQ'] + 4])
        return MolData

    def refine_data(self, optim, charge, lbcc):
        if lbcc and (charge == 0):
            lbcc_MD = self.get_ImpDat(optim, charge)
            QLBCC, DATA_Q_LJ = self.prep_lbcc(
                lbcc_MD['BONDS'], lbcc_MD['Q_LJ'])
            lbcc_MD['Q_LJ'] = DATA_Q_LJ
            BCC_file2zmat(self.zmat, QLBCC,
                          oname='/tmp/%s_BCC.z' % self.zmat[:-2])
            os.system('mv %s.z %s_NO_LBCC.z' %
                      (self.zmat[:-2], self.zmat[:-2]))
            os.system('mv %s_BCC.z %s.z' % (self.zmat[:-2], self.zmat[:-2]))
            self.MolData = lbcc_MD
        elif lbcc and (charge != 0):
            print('LBCC IS SUPPORTED ONLY FOR NEUTRAL MOLECULES')
        else:
            self.MolData = self.get_ImpDat(optim, charge)
        return None



import math
import numpy as np


def bossElement2Num(elem):
    symb2mass = {
        'H':  1,
        'B':  5,
        'C':  6,
        'N':  7,
        'O':  8,
        'F':  9,
        'Si': 14,
        'P': 15,
        'S': 16,
        'Cl': 17,
        'Br': 35,
        'I': 53,
    }
    try:
        res = symb2mass[elem]
    except NameError:
        print("Mass for atom %s is not available \n add it to symb2mass dictionary")
    return res


def pairing_func(a, b):
    ans = (a + b) * (a + b + 1) * 0.5
    if a > b:
        ans = ans + a
    else:
        ans = ans + b
    return (int(ans))


def Vector(x, y, z):
    return (x, y, z)


def length(v):
    "Return length of a vector."
    sum = 0.0
    for c in v:
        sum += c * c
    return math.sqrt(sum)


def subtract(u, v):
    "Return difference between two vectors."
    x = u[0] - v[0]
    y = u[1] - v[1]
    z = u[2] - v[2]
    return Vector(x, y, z)


def dot(u, v):
    "Return dot product of two vectors."
    sum = 0.0
    for cu, cv in zip(u, v):
        sum += cu * cv
    return sum


def Distance(u, v):
    "Return length of a vector."
#    print(u,v)
    uv = subtract(u, v)
    lsum = 0.0
    for c in uv:
        lsum += c * c
    return math.sqrt(lsum)


def cross(u, v):
    "Return the cross product of two vectors."
    x = u[1] * v[2] - u[2] * v[1]
    y = u[2] * v[0] - u[0] * v[2]
    z = u[0] * v[1] - u[1] * v[0]
    return Vector(x, y, z)


def Mol_angle(v0, v1):
    "Return angle [0..pi] between two vectors."
    cosa = round(dot(v0, v1) / length(v0) / length(v1),3)
    return np.arccos(cosa)


def angle(p0, p1, p2):
    "Return angle [0..pi] between two vectors."
    v0 = subtract(p0, p1)
    v1 = subtract(p2, p1)
    cosa = dot(v0, v1) / length(v0) / length(v1)
#    print(cosa)
    return 180.0 * np.arccos(round(cosa,3)) * 7.0 / 22.0


def dihedral(p0, p1, p2, p3):
    "Return angle [0..2*pi] formed by vertices p0-p1-p2-p3."
    v01 = subtract(p0, p1)
    v32 = subtract(p3, p2)
    v12 = subtract(p1, p2)
    v0 = cross(v12, v01)
    v3 = cross(v12, v32)
    # The cross product vectors are both normal to the axis
    # vector v12, so the angle between them is the dihedral
    # angle that we are looking for.  However, since "angle"
    # only returns values between 0 and pi, we need to make
    # sure we get the right sign relative to the rotation axis
    a = Mol_angle(v0, v3)
    if dot(cross(v0, v3), v12) > 0:
        a = -a
    return a * 180.0 * 7.0 / 22.0


def tor_id(a):
    bond = pairing_func(a[1], a[2])
    ends = pairing_func(a[0], a[3])
    return('%d-%d' % (bond, ends))


def ang_id(a):
    bond_a = pairing_func(a[0], a[1])
    bond_b = pairing_func(a[1], a[2])
    return(pairing_func(bond_a, bond_b))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
AutoZmat_VersionLSD:
A python program to create BOSS zmatrix from any molecular input format.
Need BOSS and OpenBabel executable to work
Python Modeules Needed - networkx, numpy, pandas
Created on Wed Jun 14 2017

@author: Leela Sriram Dodda
@email:  leela.dodda@yale.edu
"""
import LigParGen

import os
import numpy as np
from LigParGen.Vector_algebra import pairing_func, angle, dihedral, tor_id, ang_id,bossElement2Num, Distance
import itertools
import collections
import networkx as nx

def GenMolRep(ifile,optim,resid,charge):
    iform = ifile.split('.')
    # CREATE A MOL FILE FROM ANY FILE
    if iform[1] == 'smi': 
        os.system('babel -i%s %s -omol %s.mol --gen3D' % (iform[1], ifile, iform[0]))
    else: 
        os.system('babel -i%s %s -omol %s.mol' % (iform[1], ifile, iform[0]))
    mollines = open(iform[0] + '.mol', 'r').readlines()
    COOS, ATYPES, MolBonds = ReadMolFile(mollines)
    G_mol, mol_icords = make_graphs(ATYPES, COOS, MolBonds)
    print_ZMAT(ATYPES, G_mol, mol_icords, COOS, '%s.z' % resid, resid)
    Get_OPT('%s.z' % resid, optim, charge)
    if os.path.exists('/tmp/clu.pdb'): os.system('/bin/rm /tmp/clu.pdb')
    if iform[1] == 'pdb':
        os.system('$MCPROdir/miscexec/clu -t:f=pdb %s.pdb -r %s.z -n:f=p /tmp/clu.pdb -m ma' % (iform[0], resid))
    return(True)

def Get_OPT(zmat, optim, charge):
    assert os.path.isfile(
        zmat), 'File named %10s does not exist' % zmat
    assert 'BOSSdir' in os.environ, 'Please Make sure $BOSSdir is defined \n xZCM1A and related files are in scripts directory of BOSS'
    execs = {
        2: os.environ['BOSSdir'] + '/scripts/xZCM1A+2 > /tmp/olog',
        -2: os.environ['BOSSdir'] + '/scripts/xZCM1A-2 > /tmp/olog',
        0: os.environ['BOSSdir'] + '/scripts/xZCM1A > /tmp/olog',
        1: os.environ['BOSSdir'] + '/scripts/xZCM1A+  > /tmp/olog',
        -1: os.environ['BOSSdir'] + '/scripts/xZCM1A-  > /tmp/olog',
    }
    print('MOLECULE HAS A CHARGE of %d' % charge)
    execfile = execs[charge]
    coma = execfile + ' ' + zmat[:-2]
    os.system(coma)
    os.system('cp sum %s' % (zmat))
    execfile = os.environ['BOSSdir'] + '/scripts/xSPM > /tmp/olog'
    coma = execfile + ' ' + zmat[:-2]
    os.system(coma)
    os.system('/bin/cp sum %s' % (zmat))
    return (None)

def ReadMolFile(mollines):
    [nats, nbonds] = map(int, mollines[3].split()[0:2])
    #print([nats, nbonds])
    cooslines = mollines[4:4 + nats]
    coos = {}
    atypes = {}
    for i in range(nats):
        els = cooslines[i].split()
        coos[i + 1] = [float(e) for e in els[0:3]]
        atypes[i + 1] = els[3]
    bondlines = mollines[4 + nats:4 + nats + nbonds]
    bonds = {'BI': [], 'BJ': [], 'RIJ': [], 'UID': []}
    for line in bondlines:
        [bi, bj] = map(int, line.split()[0:2])
        bonds['BI'].append(bi)
        bonds['BJ'].append(bj)
        bonds['RIJ'].append(Distance(coos[bi], coos[bj]))
        bonds['UID'].append(pairing_func(bi, bj))
    return (coos, atypes, bonds)


def make_graphs(atoms, coos, bonds):
    G = nx.DiGraph()
    # ADD NODES USING ATOM TYPES AND COORDINATES
    for i in coos.keys():
        G.add_node(i, XYZ=coos[i], elem=atoms[i],
                   atno=bossElement2Num(atoms[i]))
    for (i, j, rij) in zip(bonds['BI'], bonds['BJ'], bonds['RIJ']):
        G.add_edge(i, j, distance=rij)
        G.add_edge(j, i, distance=rij)
    all_paths = nx.all_pairs_shortest_path(G, cutoff=3)
    all_bonds = []
    all_imps = {}
    new_tors = []
    new_angs = []
    for i in all_paths.keys():
        for j in all_paths[i].keys():
            if len(all_paths[i][j]) == 2:
                all_bonds.append(all_paths[i][j])
            elif len(all_paths[i][j]) == 3:
                new_angs.append(all_paths[i][j])
            elif len(all_paths[i][j]) == 4:
                new_tors.append(all_paths[i][j])
    dict_new_tors = {tor_id(t): t for t in new_tors}
    dict_new_angs = {ang_id(t): t for t in new_angs}
    imp_keys = [n for n in G.nodes() if G.degree(n) / 2 == 3]
    for i in imp_keys:
        nei = G.neighbors(i)
        if G.node[i]['atno'] == 6:
            all_imps[i] = [nei[0], i, nei[1], nei[2]]
    MOL_ICOORDS = {'BONDS': all_bonds,
                   'ANGLES': dict_new_angs, 'TORSIONS': dict_new_tors, 'IMPROPERS': all_imps}
    return(G, MOL_ICOORDS)


def Get_Add_Int(mol_icords, Z_BONDS, Z_ANGLES, Z_TORSIONS):
    all_bonds_mol, all_angles_mol, all_torsions_mol = mol_icords[
        'BONDS'], mol_icords['ANGLES'], mol_icords['TORSIONS']
    Z_B = {pairing_func(i[0] - 2, i[1] - 2): [i[0] - 2, i[1] - 2]
           for i in Z_BONDS.values()}
    Z_A = {ang_id([i[0] - 2, i[1] - 2, i[2] - 2]): [i[0] - 2,
                                                    i[1] - 2, i[2] - 2] for i in Z_ANGLES.values()}
    Z_T = {tor_id([i[0] - 2, i[1] - 2, i[2] - 2, i[3] - 2]): [i[0] - 2,
                                                              i[1] - 2, i[2] - 2, i[3] - 2] for i in Z_TORSIONS.values()}
    Z_Ad_B, Z_Ad_A, Z_Ad_T = collections.OrderedDict(), collections.OrderedDict(), collections.OrderedDict()
    for b_ij in all_bonds_mol:
        uid_b_ij = pairing_func(b_ij[0], b_ij[1])
        if uid_b_ij not in list(Z_B.keys()):
            Z_Ad_B[uid_b_ij] = [b_ij[0] + 2, b_ij[1] + 2]
    for a_ij in all_angles_mol.keys():
        if a_ij not in list(Z_A.keys()):
            Z_Ad_A[a_ij] = [i + 2 for i in all_angles_mol[a_ij]]
    for t_ij in all_torsions_mol.keys():
        if t_ij not in list(Z_T.keys()):
            Z_Ad_T[t_ij] = [i + 2 for i in all_torsions_mol[t_ij]]
    for c in mol_icords['IMPROPERS'].values():
        Z_Ad_T["-".join(list(map(str, c)))] = [i + 2 for i in c]
    return(Z_Ad_B, Z_Ad_A, Z_Ad_T)


def print_ZMAT(atoms, G_mol, mol_icords, coos, zmat_name, resid):
    if not zmat_name:
        zmat_name = resid
    Z_ATOMS = {1: 'X', 2: 'X'}
    Z_NO = {1: -1, 2: -1}
    Z_BONDS = {1: (1, 0, 0.000), 2: (2, 1, 1.00), 3: (3, 2, 1.00)}
    Z_ANGLES = {1: (1, 0, 0, 0.000), 2: (2, 1, 0, 0.000),
                3: (3, 2, 1, 90.00), 4: (4, 3, 2, 90.0)}
    Z_TORSIONS = {1: (1, 0, 0, 0, 0.00), 2: (2, 1, 0, 0, 0.00), 3: (
        3, 2, 1, 0, 0.00), 4: (4, 3, 2, 1, 0.00), 5: (5, 4, 3, 2, 90.0)}
    for i in range(1, len(atoms) + 1):
        Z_ATOMS[i + 2] = atoms[i]
    for i in range(1, len(atoms) + 1):
        Z_NO[i + 2] = G_mol.node[i]['atno']
    n_ats = 0
    B_LINK = {}
    for i in G_mol.nodes():
        if n_ats > 0:
            neigs = np.sort(G_mol.neighbors(i))
            B_LINK[i] = neigs[0]
            Z_BONDS[i + 2] = (i + 2, neigs[0] + 2, G_mol[i]
                              [neigs[0]]['distance'])
        n_ats += 1
    n_ats = 0
    A_LINK = {}
    for i in G_mol.nodes():
        if n_ats > 1:
            neigs = np.sort(G_mol.neighbors(B_LINK[i]))
            A_LINK[i] = neigs[0]
            ang = angle(coos[i], coos[B_LINK[i]], coos[neigs[0]])
            Z_ANGLES[i + 2] = (i + 2, B_LINK[i] + 2, neigs[0] + 2, ang)
        n_ats += 1
    n_ats = 0
    for i in G_mol.nodes():
        if n_ats > 2:
            neigs = G_mol.neighbors(A_LINK[i])
            neigs = np.array([j for j in neigs if j not in [i, B_LINK[i], A_LINK[i]]])
            neigs = np.sort(neigs)
            neigs = neigs[neigs<i]
#            print(neigs)
#            print(i,B_LINK[i], G_mol.neighbors(B_LINK[i]),A_LINK[i],G_mol.neighbors(A_LINK[i]))
            if len(neigs)<1:
#               print('IM HERE')
               neigs = [j for j in G_mol.neighbors(B_LINK[i]) if j not in [i,A_LINK[i]]]
#               print(neigs)
            [ti, tj, tk, tl] = [i, B_LINK[i], A_LINK[i], neigs[0]]
            dihed = dihedral(coos[ti], coos[tj], coos[tk], coos[tl])
            Z_TORSIONS[i + 2] = (ti + 2, tj + 2, tk + 2, tl + 2, dihed)
        n_ats += 1
    Z_Ad_B, Z_Ad_A, Z_Ad_T = Get_Add_Int(
        mol_icords, Z_BONDS, Z_ANGLES, Z_TORSIONS)
# PRINTING ACTUAL Z-MATRIX
    ofile = open(zmat_name, 'w+')
    ofile.write('BOSS Z-Matrix with LSDautozmat\n')
    for i in range(1, len(atoms) + 3):
        ofile.write('%4d %-3s%5d%5d%5d%12.6f%4d%12.6f%4d%12.6f%4s%5d\n'
                    % (i, Z_ATOMS[i], Z_NO[i], Z_NO[i], Z_BONDS[i][1], Z_BONDS[i][-1], Z_ANGLES[i][-2], Z_ANGLES[i][-1], Z_TORSIONS[i][-2], Z_TORSIONS[i][-1], resid[0:3], 1)
                    )
    ofile.write('''                    Geometry Variations follow    (2I4,F12.6)
                    Variable Bonds follow         (I4)\n''')
    for i in range(4, len(atoms) + 3):
        ofile.write('%4d\n' % i)
    ofile.write('                    Additional Bonds follow       (2I4)\n')
    if len(Z_Ad_B) > 0:
        for i in Z_Ad_B.values():
            ofile.write('%4d%4d\n' % (i[0], i[1]))
    # CREATE A FUNCTION TO DEFINE ADDITIONAL BONDS IN CASE OF RINGS
    ofile.write('''                    Harmonic Constraints follow   (2I4,4F10.4)
                    Variable Bond Angles follow   (I4)\n''')
    for i in range(5, len(atoms) + 3):
        ofile.write('%4d\n' % i)
    ofile.write('                    Additional Bond Angles follow (3I4)\n')
    if len(Z_Ad_A) > 0:
        for i in Z_Ad_A.values():
            ofile.write('%4d%4d%4d\n' % (i[0], i[1], i[2]))
    # CREATE A FUNCTION TO DEFINE ADDITIONAL BONDS IN CASE OF RINGS
    ofile.write(
        '                    Variable Dihedrals follow     (3I4,F12.6)\n')
    for i in range(6, len(atoms) + 3):
        ofile.write('%4d%4d%4d%12.6f\n' % (i, -1, -1, 0.000))
    ofile.write('                    Additional Dihedrals follow   (6I4)\n')
    if len(Z_Ad_T) > 0:
        for k in Z_Ad_T.keys():
            torsion = Z_Ad_T[k]
            ofile.write('%4d%4d%4d%4d%4d%4d\n' %
                        (torsion[0], torsion[1], torsion[2], torsion[3], -1, -1))
    ofile.write(
        '''                    Domain Definitions follow     (4I4)
                    Conformational Search (2I4,2F12.6)
                    Local Heating Residues follow (I4 or I4-I4)
                    Final blank line
''')
    ofile.close()
    return None



# THIS IS THE HEART OF BCC CORRECTION METHODOLOGY
# THIS MODULE DOES THE BCC ASSIGNMENT BY COLLECTING
# BONDING INFO AND ASSIGNING BCC CORRECTIONS FOR ATOMS
import numpy as np
import pandas as pd


def rev_bnd(bnd):
    a, b = bnd.split('-')
    return(b + '-' + a)


def sign_bnd(bnd, at):
    if (bnd == rev_bnd(bnd)):
        si = 0
    else:
        si = (-2 * bnd.split('-').index(at)) + 1
    return si


def get_bcc_types(db, cha, bond):
    rtij = []
    mtij = []
    bond['NTIJ'] = [str(cha.TY[i - 1]) + '-' + str(cha.TY[j - 1])
                    for (i, j) in zip(bond.I, bond.J)]
    for i in bond.NTIJ:
        if(i == rev_bnd(i)):
            mtij.append('X-X')
            rtij.append(i)
        elif (i in db.keys()):
            rtij.append(i)
            mtij.append(i)
        elif (rev_bnd(i) in db.keys()):
            rtij.append(rev_bnd(i))
            mtij.append(rev_bnd(i))
        else:
            print('%5s not found in bonds.csv' % i)
            mtij.append('U-U')
            rtij.append('U-U')
    bond['TIJ'] = mtij
    bond['MTIJ'] = rtij
    bond['AI'] = [str(cha.TY[i - 1]) for i in bond.I]
    bond['AJ'] = [str(cha.TY[j - 1]) for j in bond.J]
    bond['SI'] = [sign_bnd(bnd, at) for bnd, at in zip(bond.TIJ, bond.AI)]
    bond['SJ'] = [sign_bnd(bnd, at) for bnd, at in zip(bond.TIJ, bond.AJ)]
    return bond


def new_mol_info(db, cha, bond):
    #    cha = pd.read_csv('CM1AQ', header=None, delim_whitespace=True)
    #    cha.columns = ['TY', 'Q']
    bond = get_bcc_types(db, cha, bond)
    MOLBtype = {}
    for an in cha.index:
        MOLBtype[an] = list(bond[bond['I'] == an + 1].TIJ) + \
            list(bond[bond['J'] == an + 1].TIJ)
        if (cha.TY[an] == 'OS') and ('C-OS' in MOLBtype[an]):
            print("Changing OS TO OE")
            cha.loc[an, 'TY'] = 'OE'
            bond = get_bcc_types(db, cha, bond)
        # Seperate Correction for Esters
        if (cha.TY[an] == 'C') and ('C-O' in MOLBtype[an]):
            if ('C-OS' in MOLBtype[an]) or ('C-OE' in MOLBtype[an]):
                print("Changing OS TO OE")
                cha.loc[an, 'TY'] = 'CE'
                bond = get_bcc_types(db, cha, bond)
        # Seperate Correction for Amides
        if (cha.TY[an] == 'C') and ('C-N' in MOLBtype[an]):
            print("Changing C TO CAM")
            cha.loc[an, 'TY'] = 'CAM'
            bond = get_bcc_types(db, cha, bond)
        # Seperate Correction for Aromatic Nitriles
        if (cha.TY[an] == 'CZ') and (set(['CA-CZ', 'CZ-NZ']) <= set(MOLBtype[an])):
            print(MOLBtype[an])
            print("Charnging CZ-NZ to CZA-NZ")
            cha.loc[an, 'TY'] = 'CZA'
            bond = get_bcc_types(db, cha, bond)
        if (cha.TY[an] == 'CZ') and (set(['CT-CZ', 'CZ-NZ']) <= set(MOLBtype[an])):
            print(MOLBtype[an])
            print("Charnging CZ-NZ to CZT-NZ")
            cha.loc[an, 'TY'] = 'CZT'
            bond = get_bcc_types(db, cha, bond)
        # Seperate Correction for 1,2,3 Amines
        if (cha.TY[an] == 'NT') and MOLBtype[an].count('H-NT') == 2:
            print(MOLBtype[an])
            print("Charnging NT to NP")
            cha.loc[an, 'TY'] = 'NP'
            bond = get_bcc_types(db, cha, bond)
        if (cha.TY[an] == 'NT') and MOLBtype[an].count('H-NT') == 1:
            print("Charnging NT to NS")
            cha.loc[an, 'TY'] = 'NS'
            bond = get_bcc_types(db, cha, bond)
        if (cha.TY[an] == 'NT') and MOLBtype[an].count('H-NT') == 0:
            print("Charnging NT to N3")
            cha.loc[an, 'TY'] = 'N3'
            bond = get_bcc_types(db, cha, bond)
    cha = get_bcc_charges(db, bond, cha)
    QBCC = np.array(cha.QBCC)
    return(bond, cha, QBCC)


def get_bcc_charges(db, bond, cha):
    bond['IBCC'] = [sign * db[bcc] for sign, bcc in zip(bond.SI, bond.TIJ)]
    bond['JBCC'] = [sign * db[bcc] for sign, bcc in zip(bond.SJ, bond.TIJ)]
    cha['BCC'] = [sum(bond[bond['I'] == an + 1]['IBCC']) +
                  sum(bond[bond['J'] == an + 1]['JBCC']) for an in cha.index]
    cha['QBCC'] = cha['Q'] + cha['BCC']
    ars = [i for i in range(0, len(cha.TY)) if not cha['TY'][i].isdigit()]
    cha = cha.loc[ars]
    return cha


def pkl_mol_info(db, zmat):
    bond = pd.read_pickle(zmat[:-2] + '_b.pkl')
    cha = pd.read_pickle(zmat[:-2] + '_q.pkl')
    bond['IBCC'] = [sign * db[bcc] for sign, bcc in zip(bond.SI, bond.TIJ)]
    bond['JBCC'] = [sign * db[bcc] for sign, bcc in zip(bond.SJ, bond.TIJ)]
    cha['BCC'] = [sum(bond[bond['I'] == an + 1]['IBCC']) +
                  sum(bond[bond['J'] == an + 1]['JBCC']) for an in cha.index]
    cha['QBCC'] = cha['Q'] + cha['BCC']
    QBCC = np.array(cha.QBCC)
    return(bond, cha, QBCC)



from setuptools import setup, find_packages

setup(
  name = 'LigParGen',
  packages = ['LigParGen'], # this must be the same as the name above
  version = '0.11',
  description = 'Python script to provide BOSS generated OPLS-AA/CM1A(-LBCC) parameters for organic molecules and ligands.',
  author = 'Leela S. Dodda, Matthew C. Robinson',
  author_email = 'matthew.robinson@yale.edu',
  license='MIT',
  url='https://bitbucket.org/leelasd/ligpargen_2017_sep18',
  keywords = ['computational chemistry', 'force fields', 'molecular dynamics'],
  classifiers = [
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Scientific/Engineering',
        ],
  install_requires=['numpy','pandas'],
  entry_points={
        'console_scripts': [
            'converter=LigParGen.Converter:main',
        ],
    },

)




"""
SCRIPT TO CONVERT WRITE OPENMM PDB AND XML FILES 
FROM BOSS ZMATRIX
Created on Mon Feb 15 15:40:05 2016
@author: Leela S. Dodda leela.dodda@yale.edu
@author: William L. Jorgensen Lab 

Usage: python OPM_Routines.py -z phenol.z -r PHN
REQUIREMENTS:
BOSS (need to set BOSSdir in bashrc and cshrc)
Preferably Anaconda python with following modules
pandas 
argparse
numpy
"""

from LigParGen.BOSSReader import ucomb,pairing_func,bossPdbAtom2Element,bossElement2Mass
from LigParGen.BOSSReader import Refine_PDB_file,get_coos_from_pdb
#from BOSSReader import *
#from collections import OrderedDict
import pickle
import pandas as pd
import numpy as np


def GMX_pairs(tors, ang, bond):
    atom_list = np.array(list(bond.cl1) + list(bond.cl2))
    atom_list = np.sort(atom_list)
    atom_list = np.unique(atom_list)
    dict_bond = {ano: list(bond[bond.cl1 == ano]['cl2']) +
                 list(bond[bond.cl2 == ano]['cl1']) for ano in atom_list}
    dict_ang = {}
    for ano in atom_list:
        tr_1 = []
        for bno in dict_bond[ano]:
            tr_1 += dict_bond[bno]
        tr_1 = list(set(tr_1))
        tr_1.remove(ano)
        dict_ang[ano] = tr_1
    NP_dat = []
    for a in dict_bond.keys():
        for b in dict_bond[a]:
            NP_dat.append([a, b, 1, pairing_func(a + 1, b + 1)
                           [0], pairing_func(a + 1, b + 1)[1]])
            for c in dict_ang[b]:
                NP_dat.append([a, c, 3, pairing_func(a + 1, c + 1)
                               [0], pairing_func(a + 1, c + 1)[1]])
    for a in dict_ang.keys():
        for b in dict_ang[a]:
            NP_dat.append([a, b, 2, pairing_func(a + 1, b + 1)
                           [0], pairing_func(a + 1, b + 1)[1]])
            for c in dict_bond[b]:
                NP_dat.append([a, c, 3, pairing_func(a + 1, c + 1)
                               [0], pairing_func(a + 1, c + 1)[1]])
    NP_df = pd.DataFrame(NP_dat, columns=['I', 'J', 'BSEP', 'UNQ', 'TY'])
    NP_df = NP_df[NP_df.I != NP_df.J]  # CASES WITH 3 MEMBERED RINGS
    NP_df = NP_df.sort_values(['UNQ'])
    NP_B = NP_df[NP_df.BSEP == 1]
    NP_A = NP_df[NP_df.BSEP == 2]
    NP_T = NP_df[NP_df.BSEP == 3]
    NP_T = NP_T.drop_duplicates(['UNQ'])
    NP_T = NP_T[~ NP_T.UNQ.isin(NP_B.UNQ)]
    NP_T = NP_T[~ NP_T.UNQ.isin(NP_A.UNQ)]
#    NP_T.to_csv('BOND_ANGS.csv', index=False)
    return list(NP_T.TY)


def gmxDihed(df):
    #    print(
    '''
        PRINTING RYCKET-BELLMAN STYLE PARAMETERS BECAUSE FEP DOES NOT SUPPORT PROPER DIHEDRALS
    '''
#)
    [f1, f2, f3, f4] = [df.V1, df.V2, df.V3, df.V4]
    cdat = [f2 + (f1 + f3) * 0.5, 1.5 * f3 - 0.5 * f1, 4.0 *
            f4 - f2, -2.0 * f3, -4.0 * f4, 0.0, 0.00]
    return ('%5s%5s%5s%5s        3      %6.3f  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f\n' % (
        df['I'] + 1, df['J'] + 1, df['K'] + 1, df['L'] + 1, cdat[0], cdat[1], cdat[2], cdat[3], cdat[4], cdat[5]))


def gmxImp(df):
    odihed = []
    for pot in range(1, 5):
        if df['V' + str(pot)] != 0.00:
            odihed.append('%6d%6d%6d%6d    4     %10.3f %10.3f %5d  \n' % (
                df['I'] + 1, df['J'] + 1, df['K'] +
                1, df['L'] + 1, 180.00 * abs(pot % 2 - 1),
                float(df['V' + str(pot)]) * 0.5, pot))
    return (odihed)


def boss2opmTorsion(bnd_df, num2opls, st_no, molecule_data, itpf):
    dhd = []
    for line in molecule_data.MolData['TORSIONS']:
        #        dt = np.genfromtxt(line, dtype='f8')
        dt = [float(f) for f in line]
        dhd.append(dt)
    dhd = np.array(dhd)
    dhd = dhd * 4.184  # kcal to kj conversion
    dhd = dhd  # Komm = Vopls/2
    dhd_df = pd.DataFrame(dhd, columns=['V1', 'V2', 'V3', 'V4'])
    ats = []
    for line in molecule_data.MolData['ATOMS'][3:]:
        dt = [line.split()[0], line.split()[4],
              line.split()[6], line.split()[8]]
#        dt = map(int, dt)
        dt = [int(d) for d in dt]
        ats.append(dt)
    for line in molecule_data.MolData['ADD_DIHED']:
        #        dt = map(int, line)
        dt = [int(l) for l in line]
        ats.append(dt)
    assert len(ats) == len(
        dhd), 'Number of Dihedral angles in Zmatrix and Out file dont match'
    ats = np.array(ats) - st_no
    for i in range(len(ats)):
        for j in range(len(ats[0])):
            if ats[i][j] < 0:
                ats[i][j] = 0
    at_df = pd.DataFrame(ats, columns=['I', 'J', 'K', 'L'])
    final_df = pd.concat([dhd_df, at_df], axis=1, join_axes=[at_df.index])
    bndlist = list(bnd_df.UR) + (list(bnd_df.UR))
    final_df['TY'] = ['Proper' if ucomb(list([final_df.I[n], final_df.J[n], final_df.K[
        n], final_df.L[n]]), bndlist) == 3 else 'Improper' for n in range(len(final_df.I))]
    final_df['SumV'] = np.abs(
        final_df.V1) + np.abs(final_df.V2) + np.abs(final_df.V3) + np.abs(final_df.V4)
    #    final_df = final_df[final_df['SumV'] != 0.00]
    final_df['TI'] = [num2opls[j] for j in final_df.I]
    final_df['TJ'] = [num2opls[j] for j in final_df.J]
    final_df['TK'] = [num2opls[j] for j in final_df.K]
    final_df['TL'] = [num2opls[j] for j in final_df.L]
    if len(final_df.index) > 0:
        final_df['NAME'] = final_df.TI + '-' + final_df.TJ + \
            '-' + final_df.TK + '-' + final_df.TL
        final_df = final_df.sort_values(['NAME'])
#        final_df.to_csv('bos_torsions.csv', index=False)
        tor_bos = final_df.drop(
            ['I', 'J', 'K', 'L', 'TI', 'TJ', 'TK', 'TL'], 1)
        tor_bos = tor_bos.drop_duplicates()
        df = final_df.ix[tor_bos.index]
        return final_df, df
    else:
        return final_df, final_df


def boss2gmxBond(molecule_data, st_no, itpf):
    bdat = molecule_data.MolData['BONDS']
    bdat['cl1'] = [x - st_no if not x - st_no < 0 else 0 for x in bdat['cl1']]
    bdat['cl2'] = [x - st_no if not x - st_no < 0 else 0 for x in bdat['cl2']]
    bnd_df = pd.DataFrame(bdat)
    bnd_df['KIJ'] = bnd_df['KIJ'] * 836.80
    bnd_df['RIJ'] = bnd_df['RIJ'] * 0.10
    bnd_df['UNQ'] = [pairing_func(i + 1, j + 1)[0]
                     for i, j in zip(bnd_df.cl1, bnd_df.cl2)]
    bnd_df['UNQ'] = [pairing_func(i, j)[0]
                     for i, j in zip(bnd_df.cl1, bnd_df.cl2)]
    bnd_df['UF'] = ((bnd_df.cl1 + bnd_df.cl2) *
                    (bnd_df.cl1 + bnd_df.cl2 + 1) * 0.5) + bnd_df.cl2
    bnd_df['UR'] = ((bnd_df.cl1 + bnd_df.cl2) *
                    (bnd_df.cl1 + bnd_df.cl2 + 1) * 0.5) + bnd_df.cl1
    connects = []
    for (ai, aj) in zip(bnd_df.cl1, bnd_df.cl2):
        connects.append("{:>5}{:>5}    1".format(aj + 1, ai + 1))
    full_bnd = bnd_df.copy()
    bnd_df = bnd_df.drop_duplicates(['TIJ'])
    return full_bnd, connects


def boss2gmxAngle(anglefile, num2opls, st_no, itpf):
    adat = anglefile
    adat['cl1'] = [x - st_no if not x - st_no < 0 else 0 for x in adat['cl1']]
    adat['cl2'] = [x - st_no if not x - st_no < 0 else 0 for x in adat['cl2']]
    adat['cl3'] = [x - st_no if not x - st_no < 0 else 0 for x in adat['cl3']]
    ang_df = pd.DataFrame(adat)
    ang_df = ang_df[ang_df.K > 0]
    ang_df['K'] = 8.3680 * ang_df['K']
    ang_df['R'] = ang_df['R']
    full_ang = ang_df.copy()
    ang_df['TY'] = np.array([num2opls[i] + '-' + num2opls[j] + '-' + num2opls[k]
                             for i, j, k in zip(ang_df.cl1, ang_df.cl2, ang_df.cl3)])
    ang_df = ang_df.drop_duplicates(['TY'])
    return full_ang


def bossData(molecule_data):
    ats_file = molecule_data.MolData['ATOMS']
    types = []
    for i in enumerate(ats_file):
        types.append([i[1].split()[1], 'opls_' + i[1].split()[2]])
    st_no = 3
    Qs = molecule_data.MolData['Q_LJ']
    assert len(Qs) == len(types), 'Please check the at_info and Q_LJ_dat files'
    num2opls = {}
    for i in range(0, len(types)):
        num2opls[i] = Qs[i][0]
    num2typ2symb = {i: types[i] for i in range(len(Qs))}
    for i in range(len(Qs)):
        num2typ2symb[i].append(bossPdbAtom2Element(
            num2typ2symb[i][0]) + num2typ2symb[i][1][-3:])
        num2typ2symb[i].append(bossPdbAtom2Element(num2typ2symb[i][0]))
        num2typ2symb[i].append(bossElement2Mass(num2typ2symb[i][3]))
        num2typ2symb[i].append(Qs[i][0])
    return (types, Qs, num2opls, st_no, num2typ2symb)


def pdb2gro(atoms, coos, resid):
    grof = open(resid + '.gro', 'w+')
    grof.write('LIGPARGEN GENERATED GRO FILE\n')
    grof.write('%5d\n' % len(atoms))
    num = 0
    for (i, j) in zip(atoms, coos):
        num += 1
        grof.write('%5d%-5s%5s%5d%8.3f%8.3f%8.3f\n' %
                   (1, resid, i, num, j[0] * 0.1, j[1] * 0.1, j[2] * 0.1))
    grof.write('%10.5f%10.5f%10.5f\n' % (1.00000, 1.00000, 1.00000))
    grof.close()
    return None


def boss2gmxAtom(resid, num2typ2symb, Qs, itpf):
    itpf.write('[ atoms ]\n')
    itpf.write(
        ';   nr       type  resnr residue  atom   cgnr     charge       mass  \n')
    for i in range(len(Qs)):
        itpf.write(' %5d %10s %6d %6s %5s %6d %10s %10.4f \n' % (
            i + 1, num2typ2symb[i][1], 1, resid, num2typ2symb[i][0], ((i + 1) / 33.0) + 1, Qs[i][1], num2typ2symb[i][4]))
    return None


def boss2gmx(resid, molecule_data, pdb_file):
    types, Qs, num2opls, st_no, num2typ2symb = bossData(
        molecule_data)
    itpf = open(resid + '.itp', 'w+')
    itpf.write("""
;
; GENERATED BY LigParGen Server
; Jorgensen Lab @ Yale University 
;
""")
    itpf.write('[ atomtypes ]\n')
    atty = ['%10s %5s %10.4f     0.000    A    %10.5E   %10.5E\n' % (num2typ2symb[i][
        1], num2typ2symb[i][2], num2typ2symb[i][4],
        float(Qs[i][2]) * 0.1, float(Qs[i][3]) * 4.184) for
        i in range(len(Qs))]
    atty = list(set(atty))
    for att in atty:
        itpf.write('%s' % att)
    bnd_df, connects = boss2gmxBond(molecule_data, st_no, itpf)
    full_ang = boss2gmxAngle(molecule_data.MolData[
        'ANGLES'], num2opls, st_no, itpf)
    itpf.write('[ moleculetype ]\n')
    itpf.write('; Name               nrexcl\n')
    itpf.write('%s                   3\n' % resid)
    boss2gmxAtom(resid, num2typ2symb, Qs, itpf)
    itpf.write('[ bonds ]\n')
    for brow in bnd_df.iterrows():
        index, bnd = brow
        itpf.write('%5d %5d %5d %11.4f %10.3f\n' %
                   (bnd['cl1'] + 1, bnd['cl2'] + 1, 1, bnd['RIJ'], bnd['KIJ']))
    itpf.write('\n[ angles ]\n')
    itpf.write(
        ';  ai    aj    ak funct            c0            c1            c2            c3 \n')
    for row in full_ang.iterrows():
        index, angs = row
        itpf.write('%5d %5d %5d %5d %10.3f %10.3f\n' %
                   (angs['cl1'] + 1, angs['cl2'] + 1, angs['cl3'] + 1, 1, angs['R'], angs['K']))
    full_tor, tor_df = boss2opmTorsion(
        bnd_df, num2opls, st_no, molecule_data, itpf)
    if len(tor_df.index) != len(full_tor.index):
        itpf.write('\n[ dihedrals ]\n')
        itpf.write('; IMPROPER DIHEDRAL ANGLES \n')
        itpf.write(
            ';  ai    aj    ak    al funct            c0            c1            c2            c3            c4            c5\n')
#        imp_df = tor_df[tor_df.TY == 'Improper']
        for row in full_tor[full_tor.TY == 'Improper'].iterrows():
            index, tors = row
            for st in gmxImp(tors):
                itpf.write('%s' % st)
        itpf.write('\n[ dihedrals ]\n')
        itpf.write('; PROPER DIHEDRAL ANGLES\n')
        itpf.write(
            ';  ai    aj    ak    al funct            c0            c1            c2            c3            c4            c5\n')
        for row in full_tor[full_tor.TY == 'Proper'].iterrows():
            index, tors = row
#	    print tors
            for st in gmxDihed(tors):
                itpf.write('%s' % st)
#            itpf.write('%5d %5d %5d %5d %5d %10.3f %10.3f %10.3f %10.3f\n' % (
#                tors['I'] + 1, tors['J'] + 1, tors['K'] + 1, tors['L'] +
#                1, 5, tors['V1'], tors['V2'], tors['V3'],
#                tors['V4']))
    ppairs = GMX_pairs(full_tor, full_ang, bnd_df)
    itpf.write('\n[ pairs ]\n')
    for pair in ppairs:
        itpf.write('%s    1\n' % pair)
    itpf.close()
    pdblines = Refine_PDB_file(pdb_file)
    atoms, coos = get_coos_from_pdb(pdblines)
    pdb2gro(atoms, coos, resid)
    return None


def mainBOSS2GMX(resid, clu):
    mol = pickle.load(open(resid + ".p", "rb"))
    if clu:
        pdb_file = 'clu.pdb'
    else:
        pdb_file = 'plt.pdb'
    boss2gmx(resid, mol, pdb_file)
    return None
